import ddf.minim.signals.*;
import ddf.minim.*;
import ddf.minim.analysis.*;
import ddf.minim.effects.*;

Minim minim;
AudioPlayer sound;
EchoFilterFX echo;
final int sbufsize = 4096;
final int vizux = 20, vizuy = 10, vizuz = 25;
final int hrefresh = 30;
long stime = 0;
int echoval = 0;
int txtwidth = 0;
FFT fft;
CyclicBuffer[] spectrum;

class CyclicBuffer {
  private float[] b;
  int e, s;

  public CyclicBuffer(int l)
  {
    s = l;
    b = new float[l];
    e = 0;
  }

  public void addSample(float v)
  {
    e = ++e % s;
    b[e] = v;
  }

  public float getSample(int i)
  {
    return b[((s + e) - i) % s];
  }
  
  public void clear()
  {
    e = 0;
    for (int i = 0; i < s; i++)
      b[i] = 0.0f;
  }
}

class EchoFilterFX implements AudioEffect {
  CyclicBuffer hist, lhist, rhist;
  int mival, ival;

  /* Setup all required classes and buffers */
  EchoFilterFX(int maxsinterv)
  {
    hist = lhist = new CyclicBuffer(maxsinterv);
    rhist = new CyclicBuffer(maxsinterv);
    mival = maxsinterv;
    return;
  }

  /* Add a predefined interval echo to the given audiobuffer */
  void process(float[] samples)
  {
    int ival = this.ival;
    for (int i = 0; i < samples.length; i++) {
      float s = samples[i];
      s += hist.getSample(ival);
      hist.addSample(samples[i]);
      s /= 2;
      samples[i] = s;
    }
    return;
  }

  /* Calls the mono function to process a stereo echo effect */
  void process(float[] left, float[] right)
  {
    hist = lhist;
    process(left);
    hist = rhist;
    process(right);
  }

  void reset()
  {
    lhist.clear();
    rhist.clear();
  }

  void setInterval(int interval) {
    if (interval > mival)
      return;
    ival = interval;
    return;
  }
}

void setup()
{
  size(640, 480, P3D);
  frameRate(50);
  colorMode(RGB, 255);
  textFont(createFont("Times New Roman", 14));
  textAlign(LEFT, CENTER);
  textMode(SCREEN);
  txtwidth = (int)textWidth("Echo in samples: 8888.");

  minim = new Minim(this);
  sound = minim.loadFile("kiri.mp3", sbufsize);
  if (sound == null) {
    println("Could not load audio file");
    exit();
  } else {
    /* Setup visualization objects */
//    stime = System.currentTimeMillis();
    spectrum = new CyclicBuffer[vizuz];
    for (int i = 0; i < vizuz; i++)
      spectrum[i] = new CyclicBuffer(vizux + 1);

    /* Setup audio objects */
    echo = new EchoFilterFX(sbufsize);
    echo.setInterval(echoval);
    sound.addEffect(echo);
    sound.loop();

    /* Initialize a fast Fourier transform object */
    fft = new FFT(sound.bufferSize(), sound.sampleRate());
    fft.linAverages(vizuz);
  }
  return;
}

void draw()
{
  lights();
  background((int)(60 + (10 * Math.sin(2 * Math.PI *
        (System.currentTimeMillis() % 1000) / 1000.0))));

  noStroke();
  updateSpectrum();
  drawSpectrum();
  drawInterface();
}

void drawSpectrum()
{
  pushMatrix();
  translate(width / 2, height / 2, -30);
  scale(width / 16);
  translate(vizux / 2 + 1, vizuy / 2, -40 + (vizuz / 2));
  drawWall();
  translate(-1,0,0);
  for (int x = 0; x < 20; x++) {
    int z = 0, y = 0;
    for (; z < vizuz; z++) {
      y = (int)spectrum[z].getSample(x + 1);
      fill(y * 255 / vizuy, (vizuz - z) * 255 / vizuz ,40);
      translate(0, -y, 1);
      box(0.5);
      translate(0,y,0);
    }
    translate(0, 0, -z);
    translate(-1, 0, 0);
  }
  popMatrix();
}

void drawWall()
{
  int y = 0;
  for (; y < vizuy; y++) {
    int z = 0;
    for (; z < vizuz; z++) {
      translate(0, 0, 1);
      if (spectrum[z].getSample(0) > (float)y)
        fill(200);
      else
        fill(20);
      box(0.8);
    }
    translate(0, -1, -z);
  }
  translate(0, y, 0);
}

void drawInterface()
{
  fill(50,0,0);
  rect(0,height-40,width,40);
  fill(255);
  text("Echo in samples: " + echoval, 20, height - 20);
  stroke(255);
  line(20 + txtwidth, height - 20, 20 + txtwidth + 200, height - 20);
  stroke(0);
  rect(15 + txtwidth + (echoval * 200 / sbufsize), height - 30, 10, 20);
}

void updateSpectrum() {
  long curtime = System.currentTimeMillis();
  if (curtime - stime > 1000 / hrefresh) {
    stime = curtime;
    fft.forward(sound.mix);
    for (int i = 0; i < vizuz; i++) {
      int a = round(fft.getBand(i) / vizuy);
      if (a > vizuy)
        a = vizuy;
      spectrum[i].addSample(a);
    }
  }
  return;
}

void stop()
{
  if (sound != null)
    sound.close();
  minim.stop();
  super.stop();
}

void mouseDragged()
{
  int my = mouseY, mx = mouseX;
  if ((my > height - 31) && (my < height - 9)) {
    if ((mx >= 20 + txtwidth) && (mx <= 20 + txtwidth + 200)) {
      echoval = (mx - 20 - txtwidth) * (sbufsize - 1) / 200;
      echo.setInterval(echoval);
    }
  }
}
